/////////////////////////////////////////////////////////////////////////////
//
// wmp_goom.cpp : Implementation of CWmp_goom
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <ocidl.h>
#include<process.h>
#include <comutil.h>
#include <prsht.h>
#include "iwmp_goom.h"
#include "wmp_goom.h"
#include "goom_pp.h"
#include "tinyptc.h"

#include "config.h"
#include "..\..\frame_rate_tester.h"
#include "..\..\gmtimer.h"

extern "C" void goom_init (guint32 resx, guint32 resy, int cinemascope);
extern "C" void goom_set_resolution (guint32 resx, guint32 resy, int cinemascope);
extern "C" void loadbinpng(int *, int *, unsigned int ***);
extern "C" void goom_set_font(unsigned int ***, int *, int *);

/*
 * forceMode == 0 : do nothing
 * forceMode == -1 : lock the FX
 * forceMode == 1..NB_FX : force a switch to FX nforceMode
 */
extern "C" guint32 * goom_update (gint16 data [2][512], int forceMode, float fps, char *songTitle);

extern "C" void goom_close ();

#define DEFXVEL 320
#define DEFYVEL 240
#define DEFCSCOPE 0

bool needreg;
bool global_bitmapInit = false;
CWmp_goom *global_goom = NULL;

class RegSettings {
public:
	RegSettings();
	~RegSettings();

	void Read();
	void Write();

	bool showfps;
	bool showtitle;
	float sleeprate;
	int surface_scope;

};
RegSettings::RegSettings()
{
	surface_scope = DEFCSCOPE;
	showfps = true;
	showtitle = true;
	sleeprate=1.0f/30.0f;
	Read();
}
RegSettings::~RegSettings()
{
}
void RegSettings::Write()
{
	HKEY hKey; DWORD disp;
	LONG res = RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\Goom\\WMP",0,NULL,0,KEY_WRITE,NULL,&hKey,&disp);
	if (res!=ERROR_SUCCESS) return;
	RegSetValueEx(hKey,"surface_scope",0,REG_BINARY,(LPBYTE)&surface_scope,sizeof(surface_scope));
	RegSetValueEx(hKey,"showfps",0,REG_BINARY,(LPBYTE)&showfps,sizeof(showfps));
	RegSetValueEx(hKey,"showtitle",0,REG_BINARY,(LPBYTE)&showtitle,sizeof(showtitle));
	RegSetValueEx(hKey,"sleeprate",0,REG_BINARY,(LPBYTE)&sleeprate,sizeof(sleeprate));
	RegCloseKey(hKey);
}
void RegSettings::Read()
{
	DWORD cbData;
	HKEY hKey;
	LONG res = RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Goom\\WMP",0,KEY_READ,&hKey);
	if (res!=ERROR_SUCCESS) return;
	cbData=sizeof(surface_scope);RegQueryValueEx(hKey,"surface_scope",0,NULL,(LPBYTE)&surface_scope,&cbData);
	cbData=sizeof(showfps);RegQueryValueEx(hKey,"showfps",0,NULL,(LPBYTE)&showfps,&cbData);
	cbData=sizeof(showtitle);RegQueryValueEx(hKey,"showtitle",0,NULL,(LPBYTE)&showtitle,&cbData);
	cbData=sizeof(sleeprate);RegQueryValueEx(hKey,"sleeprate",0,NULL,(LPBYTE)&sleeprate,&cbData);
	RegCloseKey(hKey);

}
///////////////////////////////////////////////
class SleepMgr {
	bool inWait;
	HANDLE hev;
public:
	SleepMgr();
	~SleepMgr();
	void Sleep();
	void Wakeup();
};

SleepMgr::SleepMgr()
{
	inWait=false;
	hev=CreateEvent(NULL,TRUE,FALSE,NULL);
}
SleepMgr::~SleepMgr()
{
	if (hev != NULL) CloseHandle(hev);
}
void SleepMgr::Sleep()
{
	if (hev != NULL) {
		inWait = true;
		OutputDebugString("Goom is Sleeping\n");
		WaitForSingleObject(hev,INFINITE);
		inWait=false;
		ResetEvent(hev);
		OutputDebugString("Goom is awake\n");
	} else {
		::Sleep(1000);
	}
}
void SleepMgr::Wakeup()
{
	if (hev != NULL) {
		if (inWait) {
			OutputDebugString("Goom should wake up\n");
			SetEvent(hev);
		}
	}
}


/////////////////////////////////////////////////////////////////////////////
// CWmp_goom::CWmp_goom
// Constructor

CWmp_goom::CWmp_goom() :
m_clrForeground(0x0000FF),
m_nPreset(0)
{
#ifdef _DEBUG
	char tmpch[256];
	sprintf(tmpch,"GOOM::Construct %08x\n",this);
	OutputDebugString(tmpch);
#endif
	bitmapInit=false;
	surface_width = DEFXVEL;
	surface_height = DEFYVEL;
	bFullScreen = false;
	bSetRender=false;
	render_wnd = NULL;
	bQuit=true;
	songTitle1=NULL;
	songSave=NULL;
	songSave_mem=NULL;
	int i;
	for (i = 0; i < 0x100; i++) {
		pngbuf[i] =NULL;
	}
	fontloaded=false;
	parent_wnd = NULL;
	needreg=false;
	regset = new RegSettings();
	sleeper = NULL;
	cacheHwnd = NULL;
	dorender=false;
	lastxr = -1;
	lastyr = -1;
	lastcs = -1;
	xgoom_first = true;
	goomOwner=false;
}

/////////////////////////////////////////////////////////////////////////////
// CWmp_goom::~CWmp_goom
// Destructor

CWmp_goom::~CWmp_goom()
{
#ifdef _DEBUG
	char tmpch[256];
	sprintf(tmpch,"GOOM::Destruct %08x\n",this);
	OutputDebugString(tmpch);
#endif
	if (bitmapInit) {
		if (dorender) {
			bQuit=true;
			WaitForSingleObject(hThread,INFINITE);
		} else {
//			if (goomOwner) framerate_tester_close();
		}
//		if (goomOwner) {
//			goom_close();
//			global_bitmapInit = false;
//		}
	}
	int i,yy;
	for (i = 0; i < 0x100; i++) {
		if (pngbuf[i] == NULL) continue;
		if (pngbuf[i] == pngbuf[42] && i != 42) continue;
		for (yy = 0; yy < pngh[i]; yy++) {
			free(pngbuf[i][yy]);
		}
		free(pngbuf[i]);
	}
	if (songTitle1 != NULL) delete[] songTitle1;
	if (songSave_mem != NULL) delete[] songSave_mem;
	if (regset != NULL) delete regset;
	if (sleeper != NULL) delete sleeper;
}

/////////////////////////////////////////////////////////////////////////////
// CWmp_goom:::FinalConstruct
// Called when an effect is first loaded. Use this function to do one-time
// intializations that could fail (i.e. creating offscreen buffers) instead
// of doing this in the constructor, which cannot return an error.

HRESULT CWmp_goom::FinalConstruct()
{
    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWmp_goom:::FinalRelease
// Called when an effect is unloaded. Use this function to free any
// resources allocated in FinalConstruct.

void CWmp_goom::FinalRelease()
{
#ifdef _DEBUG
	char tmpch[256];
	sprintf(tmpch,"GOOM::FinalRelease %08x\n",this);
	OutputDebugString(tmpch);
#endif
	if (bitmapInit) {
		if (dorender) {
		} else {
			if (goomOwner) framerate_tester_close();
		}
		if (goomOwner) {
			goom_close();
			global_bitmapInit = false;
			global_goom = NULL;
			goomOwner=false;
		}
		bitmapInit = false;
	}
}

void CWmp_goom::LoadFont()
{
	if (!fontloaded) {
		int i;
		loadbinpng(pngw,pngh,pngbuf);
		for (i = 0; i < 0x100; i++) {
			if (pngbuf[i] == NULL) {
				pngbuf[i] = pngbuf[42];
				pngw[i] = pngw[42];
				pngh[i] = pngh[42];
			}
		}
		fontloaded=true;
	}
}

static short s1 = 128;
static short s2 = 256;
static gint16 data[2][512];
static gint16 data2[2][512];
static int xgoom_render_pcm(CWmp_goom *pGoom,int idx, char *title);
#define MAX_FRAMERATE (32.0f)
#define INTERPIX (1.0f/MAX_FRAMERATE)

static void __cdecl render_thread(void *pVoid)
{
  float time1,time2;
  int i,j;
  static gint16 prev0 = 0;
  static int prev1 = 0;
  BOOL bError = FALSE;
  CWmp_goom *pGoom = (CWmp_goom *)pVoid;
  float spf = INTERPIX;

  GMTimer *t = gmtimer_new();
  time1 = gmtimer_getvalue (t);
  framerate_tester_init();

  ZeroMemory(&data[0][0],sizeof(data));

  if (pGoom->dorender) {
    ptc_set_hwnd(pGoom->render_wnd);
	if (!ptc_open("What A GOOM!",pGoom->surface_width,pGoom->surface_height)) {
		  bError = TRUE;
	}
  }
	bool bIcon=false;

  while (!pGoom->bQuit && !bError) {

	/* main work */
	if (prev0 == data [0][0]) {
		if (prev1 == 0) {
			prev1 = 512;
		} else {
		  for (i=0;i<2;i++)
			for (j=0;j<512;j++)
			  data2 [i][j] = (data2 [i][j] * 31) >> 5;
		}
	} else {
		prev1 = 0;
	}
	prev0 = data [0][0];
	if (pGoom->xgoom_render_pcm (prev1,NULL,NULL) == 1) {
		bError = TRUE;
		break;
	}
	/**/

	time2 = gmtimer_getvalue (t);
	if (pGoom->regset->sleeprate > 0.0 || bIcon) {
		if (bIcon) {
			pGoom->sleeper->Sleep();
		} else {
			spf = pGoom->regset->sleeprate;
			if ((time2 - time1) < spf) {
				DWORD ti = (DWORD)(1000.0*(spf + time1 - time2));
				Sleep (ti);
				time1 = time1 + spf;
			} else
				time1 = time2;
		}
	} else { 
		  time1 = time2;
	}

	bIcon=false;
	if (pGoom->parent_wnd != NULL) {
	  if (::IsIconic(pGoom->parent_wnd)) {
		  bIcon=true;
	  } 
	}

  }
  gmtimer_delete(&t);
  framerate_tester_close();
  if (pGoom->dorender) ptc_close();
  if (bError) {
	  while (!pGoom->bQuit) {
		  Sleep(1000);
	  }
  }
}

int CWmp_goom::xgoom_render_pcm(int idx, HDC hdc, RECT *prc)
{
	bool bChangeUpdate;

	if (songSave_mem == NULL) {
		if (songTitle1 != NULL) {
			songSave_mem = new char[strlen(songTitle1)+1];
			strcpy(songSave_mem,songTitle1);
			if (songTitle1 != NULL) delete[] songTitle1;
			songTitle1=NULL;
			songSave=songSave_mem;
		}
	} else {
		if (songTitle1 != NULL) {
			if (strcmp(songSave_mem,songTitle1)) {
				delete[] songSave_mem;
				songSave_mem = new char[strlen(songTitle1)+1];
				strcpy(songSave_mem,songTitle1);
				if (songTitle1 != NULL) delete[] songTitle1;
				songTitle1=NULL;
				songSave=songSave_mem;
			}
		}
	}

	bChangeUpdate = ((lastxr != surface_width) || (lastyr != surface_height) || (lastcs != regset->surface_scope));
	
//	if (pGoom->bChangeFS && !pGoom->bFullScreen && (pGoom->render_wnd == NULL)) {
	if (bChangeFS && !bSetRender) {
		// need to wait until CWmp_goom::Render is called again
		return 0;
	}


	if (bChangeUpdate || bChangeFS) {
		if (!xgoom_first) {
			if (dorender) ptc_close ();
			if (bChangeUpdate) {
				goom_set_resolution (surface_width,surface_height,regset->surface_scope);
				LoadFont();
				goom_set_font(pngbuf,pngw,pngh);
			}
			if (dorender) {
				ptc_set_hwnd(render_wnd);
				if (!ptc_open("What a GOOM!",
						surface_width, surface_height))
						      return 1;
			} else {
				bitmap_header->bmiHeader.biWidth = surface_width;
				bitmap_header->bmiHeader.biHeight = -surface_height;	// note well
			}
		}
		xgoom_first = false;
		lastxr = surface_width;
		lastyr = surface_height;
		lastcs = regset->surface_scope;
		bChangeFS = false;
	}

	float fps = framerate_tester_getvalue();
	if (!regset->showfps) fps=-1.0f;
	if (!regset->showtitle) songSave=NULL;

	unsigned int *buf;
	if (idx == 0) {
		buf = goom_update (data,0,fps,songSave);
	} else {
		buf = goom_update (data2,0,fps,songSave);
	}
	if (dorender) {
		ptc_update(buf);
	} else {
		int offx, offy;
		offx = (surface_width - (prc->right - prc->left) )>>1;
		offy = (surface_height - (prc->bottom - prc->top)  )>>1;
		if (offx < 0) offx=0;
		if (offy < 0) offy=0;
	    StretchDIBits(hdc, prc->left, prc->top, prc->right-prc->left, prc->bottom-prc->top, offx, offy, surface_width-(offx<<1), 
              surface_height-(offy<<1), buf, bitmap_header, DIB_RGB_COLORS, SRCCOPY);
	}
	framerate_tester_newframe();
	songSave=NULL;

	return 0;
}


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::Render
// Called when an effect should render itself to the screen.
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::Render(TimedLevel *pLevels, HDC hdc, RECT *prc)
{
	if (global_goom != NULL && this != global_goom) return global_goom->Render(pLevels,hdc,prc);

	if (pLevels->state == 0) { // return and cleanup on stop

		FinalRelease();

		HBRUSH hNewBrush = ::CreateSolidBrush( RGB(0,0,0) );
		::FillRect( hdc, prc, hNewBrush );

	    if (hNewBrush) {
		    ::DeleteObject( hNewBrush );
	    }
		return S_OK;
	}

	if (dorender) {
		render_wnd = ::WindowFromDC(hdc);
		if (parent_wnd == NULL) {
			HWND htmp = render_wnd;
			parent_wnd = render_wnd;
			while ( (htmp = ::GetParent(htmp)) != NULL) {
				parent_wnd = htmp;
			}
		}
		if (cacheHwnd != render_wnd) {
			cacheHwnd=render_wnd;
			OutputDebugString("GOOM::Render window changed\n");
		}
	}
	bSetRender=true;

	if (needreg) {
		needreg=false;
		regset->Read();
	}
	if (sleeper != NULL) sleeper->Wakeup();

	if (pLevels->state == 1) { // return on pause
		if (!dorender) {
			if (bitmapInit) {
				float fps = framerate_tester_getvalue();
				framerate_tester_newframe();
			}
		}
		return S_OK;
	}

	if (!bitmapInit || !global_bitmapInit) { 
#ifdef _DEBUG
	char tmpch[256];
	sprintf(tmpch,"GOOM::BitmapInit %08x\n",this);
	OutputDebugString(tmpch);
#endif
		surface_width=DEFXVEL;
		surface_height=DEFYVEL;
		SetCurrentPreset(m_nPreset);
		bFullScreen=false;
		bChangeFS = false;
		bitmapInit = true;
		if (sleeper != NULL) delete sleeper;
		sleeper = new SleepMgr();
		// check global Flag cause there can only be one Goom
		goomOwner=!global_bitmapInit;
		if (goomOwner) {
			goom_init(surface_width,surface_height,regset->surface_scope);
			if (!dorender) framerate_tester_init();
			global_bitmapInit = true;
			LoadFont();
			goom_set_font(pngbuf,pngw,pngh);
		}
		global_bitmapInit=true;
		global_goom = this;
		bQuit = false;
		if (dorender) {
			hThread = (HANDLE)_beginthread(render_thread,0,(void *)this);
			if ((int)hThread == -1) {
				return E_FAIL;
			}
		} else {
			int cc;
			for (cc = 0; cc < sizeof(BITMAPINFOHEADER)+260*4; cc++)
				bitmapbuffer[cc] = 0;
			bitmap_header = (BITMAPINFO *)&bitmapbuffer;
			bitmap_header->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
			bitmap_header->bmiHeader.biPlanes = 1;
			bitmap_header->bmiHeader.biBitCount = 32;
			bitmap_header->bmiHeader.biCompression = BI_BITFIELDS;
			bitmap_header->bmiHeader.biWidth = surface_width;
			bitmap_header->bmiHeader.biHeight = -surface_height;	// note well
			((unsigned long *)bitmap_header->bmiColors)[0] = 0x00FF0000;
			((unsigned long *)bitmap_header->bmiColors)[1] = 0x0000FF00;
			((unsigned long *)bitmap_header->bmiColors)[2] = 0x000000FF;
		}
	}

//	if (!goomOwner) return S_OK;

	int i;
	for (i = 0; i < 512; i++) {
		data[0][i] = ((short)pLevels->waveform[0][i]^s1) * s2;
		data[1][i] = ((short)pLevels->waveform[1][i]^s1) * s2;
	}
	if (dorender) {
		for (i = 0; i < 512; i++) {
			data2[0][i] = ((short)pLevels->waveform[0][i+512]^s1) * s2;
			data2[1][i] = ((short)pLevels->waveform[1][i+512]^s1) * s2;
		}
	}

	if (!dorender) {
		xgoom_render_pcm(0,hdc,prc);
	}
    
    return S_OK;
}

STDMETHODIMP CWmp_goom::GoFullscreen(BOOL fFullScreen)
{
	bSetRender=false;
	bChangeFS=true;
	if (fFullScreen) {
//		render_wnd = ::GetTopWindow(NULL);
		bFullScreen = true;
	} else {
		render_wnd = NULL;
		bFullScreen = false;
	}

	return S_OK;
}
STDMETHODIMP CWmp_goom::RenderFullScreen(TimedLevel *pLevels)
{
	int i;
	parent_wnd = NULL;
    render_wnd = ::GetTopWindow(NULL);
    RECT r;
	r.left=0;
	r.right=0;
	if (render_wnd != NULL) {
       ::GetClientRect(render_wnd,&r);
	}
    while(r.left == r.right) {
        render_wnd = ::GetNextWindow(render_wnd,GW_HWNDNEXT);
        if (render_wnd == NULL) {
            render_wnd = ::GetTopWindow(NULL);
            break;
        }
        ::GetClientRect(render_wnd,&r);
    }
    bSetRender=true;

	if (sleeper != NULL) sleeper->Wakeup();

	for (i = 0; i < 512; i++) {
		data[0][i] = ((short)pLevels->waveform[0][i]^s1) * s2;
		data[1][i] = ((short)pLevels->waveform[1][i]^s1) * s2;
		data2[0][i] = ((short)pLevels->waveform[0][i+512]^s1) * s2;
		data2[1][i] = ((short)pLevels->waveform[1][i+512]^s1) * s2;
	}
   return S_OK;
}
//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::MediaInfo
// Everytime new media is loaded, this method is called to pass the
// number of channels (mono/stereo), the sample rate of the media, and the
// title of the media
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::MediaInfo(LONG lChannelCount, LONG lSampleRate, BSTR bstrTitle )
{
	songTitle1 = _com_util::ConvertBSTRToString(bstrTitle);
    return S_OK;
}

extern "C" char readme_wmp[];

static BOOL APIENTRY PropDlgProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	CWmp_goom *pGoom = NULL;
  if (msg!=WM_INITDIALOG) pGoom = (CWmp_goom*)GetWindowLong(hwnd,DWL_USER);
  if (msg!=WM_INITDIALOG && pGoom==NULL) return FALSE;
  switch (msg) 
  { case WM_INITDIALOG:
    { 
		PROPSHEETPAGE *psp = (PROPSHEETPAGE*)lParam;
      pGoom = (CWmp_goom*) (psp->lParam);
      SetWindowLong(hwnd,DWL_USER,(LONG)pGoom);
	  if (pGoom == NULL) {
		  HWND hEdit = GetDlgItem(hwnd,IDC_EDIT_ABOUT);
		  if (hEdit != NULL) {
			  SendMessage(hEdit,WM_SETTEXT,0,(LPARAM)readme_wmp);
		  }
	  } else {
		  HWND hlim = GetDlgItem(hwnd,IDC_COMBO_FPSLIM);
		  ShowWindow(hlim,SW_HIDE);
		  HWND hlimlab = GetDlgItem(hwnd,IDC_STATIC_FRLIM);
		  ShowWindow(hlimlab,SW_HIDE);
		  if (hlim != NULL) {
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"5");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"10");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"15");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"20");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"25");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"30");
			  SendMessage(hlim,CB_ADDSTRING,0,(LPARAM)"None");
			  if (pGoom->regset->sleeprate == (1.0f/5.0f)) SendMessage(hlim,CB_SETCURSEL,0,0);
			  else if (pGoom->regset->sleeprate == (1.0f/10.0f)) SendMessage(hlim,CB_SETCURSEL,1,0);
			  else if (pGoom->regset->sleeprate == (1.0f/15.0f)) SendMessage(hlim,CB_SETCURSEL,2,0);
			  else if (pGoom->regset->sleeprate == (1.0f/20.0f)) SendMessage(hlim,CB_SETCURSEL,3,0);
			  else if (pGoom->regset->sleeprate == (1.0f/25.0f)) SendMessage(hlim,CB_SETCURSEL,4,0);
			  else if (pGoom->regset->sleeprate == (1.0f/30.0f)) SendMessage(hlim,CB_SETCURSEL,5,0);
			  else SendMessage(hlim,CB_SETCURSEL,6,0);
		  }
		  HWND htmp = GetDlgItem(hwnd,IDC_CHECK_FPS);
		  if (pGoom->regset->showfps) {
			  SendMessage(htmp,BM_SETCHECK,BST_CHECKED,0);
		  } else {
			  SendMessage(htmp,BM_SETCHECK,BST_UNCHECKED,0);
		  }
		  htmp = GetDlgItem(hwnd,IDC_CHECK_TITLE);
		  if (pGoom->regset->showtitle) {
			  SendMessage(htmp,BM_SETCHECK,BST_CHECKED,0);
		  } else {
			  SendMessage(htmp,BM_SETCHECK,BST_UNCHECKED,0);
		  }
		  htmp = GetDlgItem(hwnd,IDC_CHECK_CSCOPE);
		  if (pGoom->regset->surface_scope != 0) {
			  SendMessage(htmp,BM_SETCHECK,BST_CHECKED,0);
		  } else {
			  SendMessage(htmp,BM_SETCHECK,BST_UNCHECKED,0);
		  }
	  }
      return TRUE;
    }
    case WM_COMMAND:
    {
		/*
	WORD id = LOWORD(wParam);
      if (id==ID_EDIT)
      {
        return TRUE;
      }
	  */
      return FALSE;
    }
    case WM_NOTIFY:
    {
		int code=(((NMHDR FAR *) lParam)->code);
		if (code == PSN_APPLY && pGoom != NULL) {
			int ichk;
		  HWND hlim = GetDlgItem(hwnd,IDC_COMBO_FPSLIM);
		  if (hlim != NULL) {
			  ichk=SendMessage(hlim,CB_GETCURSEL,0,0);
			  switch (ichk) {
			  case 0:
				  pGoom->regset->sleeprate = 1.0f/5.0f;
				  break;
			  case 1:
				  pGoom->regset->sleeprate = 1.0f/10.0f;
				  break;
			  case 2:
				  pGoom->regset->sleeprate = 1.0f/15.0f;
				  break;
			  case 3:
				  pGoom->regset->sleeprate = 1.0f/20.0f;
				  break;
			  case 4:
				  pGoom->regset->sleeprate = 1.0f/25.0f;
				  break;
			  case 5:
				  pGoom->regset->sleeprate = 1.0f/30.0f;
				  break;
			  case 6:
				  pGoom->regset->sleeprate = -1.0f;
				  break;
			  }
		  }
		  HWND htmp = GetDlgItem(hwnd,IDC_CHECK_FPS);
		  if (htmp != NULL) pGoom->regset->showfps = (SendMessage(htmp,BM_GETCHECK,0,0) == BST_CHECKED);
		  htmp = GetDlgItem(hwnd,IDC_CHECK_TITLE);
		  if (htmp != NULL) pGoom->regset->showtitle = (SendMessage(htmp,BM_GETCHECK,0,0) == BST_CHECKED);
		  htmp = GetDlgItem(hwnd,IDC_CHECK_CSCOPE);
		  if (htmp != NULL) {
			  if (SendMessage(htmp,BM_GETCHECK,0,0) == BST_CHECKED) {
				  pGoom->regset->surface_scope = 1;
			  } else {
				  pGoom->regset->surface_scope = 0;
			  }
		  }
		  pGoom->regset->Write();
		  needreg=true;
		} 
      return TRUE;
    }
    case WM_DESTROY:
    {
      return TRUE;
    }
  }
  return FALSE;
}


STDMETHODIMP CWmp_goom::DisplayPropertyPage(HWND hwndOwner)
{
 PROPSHEETPAGE psp[2];
  PROPSHEETHEADER psh;
//  TPropStuff ps; ps.sticky=this;
  psp[0].dwSize = sizeof(PROPSHEETPAGE);
//  psp[0].dwFlags = PSP_USEICONID | PSP_USETITLE;
  psp[0].dwFlags = PSP_USETITLE;
  psp[0].hInstance = _Module.m_hInst;
  psp[0].pszTemplate = MAKEINTRESOURCE(IDD_GOOM_PP);
  psp[0].pszIcon = 0;
  psp[0].pfnDlgProc = PropDlgProc;
  psp[0].pszTitle = "Settings";
//  psp[0].lParam = (LPARAM)&ps;
  psp[0].lParam=(LPARAM)this;
  psp[0].pfnCallback = NULL;
  psp[1].dwSize = sizeof(PROPSHEETPAGE);
//  psp[0].dwFlags = PSP_USEICONID | PSP_USETITLE;
  psp[1].dwFlags = PSP_USETITLE;
  psp[1].hInstance = _Module.m_hInst;
  psp[1].pszTemplate = MAKEINTRESOURCE(IDD_GOOM_ABOUT);
  psp[1].pszIcon = 0;
  psp[1].pfnDlgProc = PropDlgProc;
  psp[1].pszTitle = "About";
//  psp[0].lParam = (LPARAM)&ps;
  psp[1].lParam=NULL;
  psp[1].pfnCallback = NULL;
  psh.dwSize = sizeof(PROPSHEETHEADER);
//  psh.dwFlags = PSH_USEICONID | PSH_PROPSHEETPAGE;
  psh.dwFlags = PSH_PROPSHEETPAGE;
  psh.hwndParent = hwndOwner;
//  psh.hInstance = _Module.m_hInst;
  psh.hInstance = psp[0].hInstance;
//  psh.pszIcon = MAKEINTRESOURCE(2);
  psh.pszIcon = 0;
  psh.pszCaption = (LPSTR) "Goom! Visualization";
  psh.nPages = sizeof(psp) / sizeof(PROPSHEETPAGE);
  psh.nStartPage = 0;
  psh.ppsp = (LPCPROPSHEETPAGE) &psp;
  psh.pfnCallback = NULL;
  PropertySheet(&psh);
	 return S_OK;
}


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::GetCapabilities
// Returns the capabilities of this effect. Flags that can be returned are:
//	EFFECT_CANGOFULLSCREEN		-- effect supports full-screen rendering
//	EFFECT_HASPROPERTYPAGE		-- effect supports a property page
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::GetCapabilities(DWORD * pdwCapabilities)
{
    if (NULL == pdwCapabilities)
    {
        return E_POINTER;
    }

    *pdwCapabilities = EFFECT_HASPROPERTYPAGE;
	if (dorender)  *pdwCapabilities |= EFFECT_CANGOFULLSCREEN;

	if (*pdwCapabilities & EFFECT_HASPROPERTYPAGE) {
		OutputDebugString("Able to do a propertypage\n");
	}
	if (*pdwCapabilities & EFFECT_CANGOFULLSCREEN) {
		OutputDebugString("Able to go fullscreen\n");
	}
    return S_OK;
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::GetTitle
// Invoked when a host wants to obtain the title of the effect
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::GetTitle(BSTR* bstrTitle)
{
    USES_CONVERSION;

    if (NULL == bstrTitle)
    {
        return E_POINTER;
    }

    CComBSTR bstrTemp;
    bstrTemp.LoadString(IDS_EFFECTNAME); 
        
    if ((!bstrTemp) || (0 == bstrTemp.Length))
    {
        return E_FAIL;
    }

    *bstrTitle = bstrTemp.Detach();

    return S_OK;
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::GetPresetTitle
// Invoked when a host wants to obtain the title of the given preset
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::GetPresetTitle(LONG nPreset, BSTR *bstrPresetTitle)
{
	USES_CONVERSION;

    if (NULL == bstrPresetTitle)
    {
        return E_POINTER;
    }

    if ((nPreset < 0) || (nPreset >= PRESET_COUNT))
    {
        return E_INVALIDARG;
    }

    CComBSTR bstrTemp;
    
    switch (nPreset)
    {
    case PRESET_320_240:
        bstrTemp.LoadString(IDS_320_240); 
        break;
    case PRESET_480_360:
        bstrTemp.LoadString(IDS_480_360); 
        break;
    case PRESET_640_480:
        bstrTemp.LoadString(IDS_640_480); 
        break;
    case PRESET_800_600:
        bstrTemp.LoadString(IDS_800_600); 
        break;
    }
    
    if ((!bstrTemp) || (0 == bstrTemp.Length))
    {
        return E_FAIL;
    }

    *bstrPresetTitle = bstrTemp.Detach();

    return S_OK;
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::GetPresetCount
// Invoked when a host wants to obtain the number of supported presets
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::GetPresetCount(LONG *pnPresetCount)
{
    if (NULL == pnPresetCount)
    {
        return E_POINTER;
    }

    *pnPresetCount = PRESET_COUNT;

    return (S_OK);
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::SetCurrentPreset
// Invoked when a host wants to change the index of the current preset
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::SetCurrentPreset(LONG nPreset)
{
    if ((nPreset < 0) || (nPreset >= PRESET_COUNT))
    {
        return E_INVALIDARG;
    }

    m_nPreset = nPreset;

    switch (nPreset)
    {
    case PRESET_320_240:
		surface_width=320;
		surface_height=240;
        break;
    case PRESET_480_360:
		surface_width=480;
		surface_height=360;
        break;
    case PRESET_640_480:
		surface_width=600;
		surface_height=480;
        break;
    case PRESET_800_600:
		surface_width=800;
		surface_height=600;
        break;
    }

    return (S_OK);
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::GetCurrentPreset
// Invoked when a host wants to obtain the index of the current preset
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::GetCurrentPreset(LONG *pnPreset)
{
    if (NULL == pnPreset)
    {
        return E_POINTER;
    }

    *pnPreset = m_nPreset;

    return (S_OK);
}

//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::get_foregroundColor
// Property get to retrieve the foregroundColor prop via the public interface.
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::get_foregroundColor(BSTR *pVal)
{
	return ColorToWz( pVal, m_clrForeground);
}


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::put_foregroundColor
// Property put to set the foregroundColor prop via the public interface.
//////////////////////////////////////////////////////////////////////////////
STDMETHODIMP CWmp_goom::put_foregroundColor(BSTR newVal)
{
	return WzToColor(newVal, &m_clrForeground);
}


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::WzToColor
// Helper function used to convert a string into a COLORREF.
//////////////////////////////////////////////////////////////////////////////
HRESULT CWmp_goom::WzToColor(const WCHAR *pwszColor, COLORREF *pcrColor)
{ 
    if (NULL == pwszColor)
    {
        //NULL color string passed in
        return E_POINTER;
    }

    if (0 == lstrlenW(pwszColor))
    {
        //Empty color string passed in
        return E_INVALIDARG;
    }

    if (NULL == pcrColor)
    {
        //NULL output color DWORD passed in
        return E_POINTER;
    }
    
    if (lstrlenW(pwszColor) != 7)
    {
        //hex color string is not of the correct length
        return E_INVALIDARG;
    }

    DWORD dwRet = 0;
    for (int i = 1; i < 7; i++)
    {
        // shift dwRet by 4
        dwRet <<= 4;
        // and add in the value of this string

        if ((pwszColor[i] >= L'0') && (pwszColor[i] <= L'9'))
        {
            dwRet += pwszColor[i] - '0';
        }
        else if ((pwszColor[i] >= L'A') && (pwszColor[i] <= L'F'))
        {
            dwRet += 10 + (pwszColor[i] - L'A');
        }
        else if ((pwszColor[i] >= L'a') && (pwszColor[i] <= L'f'))
        {
            dwRet += 10 + (pwszColor[i] - L'a');
        }
        else
        {
           //Invalid hex digit in color string
            return E_INVALIDARG;
        }
    }

    *pcrColor = SwapBytes(dwRet);

    return S_OK;
} 


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::ColorToWz
// Helper function used to convert a COLORREF to a BSTR.
//////////////////////////////////////////////////////////////////////////////
HRESULT CWmp_goom::ColorToWz( BSTR* pbstrColor, COLORREF crColor)
{
    _ASSERT( NULL != pbstrColor );
    _ASSERT( (crColor & 0x00FFFFFF) == crColor );

    *pbstrColor = NULL;

    WCHAR wsz[8];
    HRESULT hr  = S_OK;

    wsprintfW( wsz, L"#%06X", SwapBytes(crColor) );
    
    *pbstrColor = ::SysAllocString( wsz );

    if (!pbstrColor)
    {
        hr = E_FAIL;
    }

    return hr;
}


//////////////////////////////////////////////////////////////////////////////
// CWmp_goom::SwapBytes
// Used to convert between a DWORD and COLORREF.  Simply swaps the lowest 
// and 3rd order bytes.
//////////////////////////////////////////////////////////////////////////////
inline DWORD CWmp_goom::SwapBytes(DWORD dwRet)
{
    return ((dwRet & 0x0000FF00) | ((dwRet & 0x00FF0000) >> 16) | ((dwRet & 0x000000FF) << 16));
}

